import tensorflow as tf
from transformers_keras.modeling_bert import BertModel, BertPretrainedModel


class BertForSentenceEmbedding(BertPretrainedModel):
    """Bert for sentence embedding."""

    def __init__(
        self,
        vocab_size=21128,
        max_positions=512,
        hidden_size=768,
        type_vocab_size=2,
        num_layers=6,
        num_attention_heads=8,
        intermediate_size=3072,
        activation="gelu",
        hidden_dropout_rate=0.2,
        attention_dropout_rate=0.1,
        initializer_range=0.02,
        epsilon=1e-12,
        **kwargs
    ):
        # build functional model
        input_ids = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name="input_ids")
        segment_ids = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name="segment_ids")
        attention_mask = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name="attention_mask")
        bert_model = BertModel(
            vocab_size=vocab_size,
            max_positions=max_positions,
            hidden_size=hidden_size,
            type_vocab_size=type_vocab_size,
            num_layers=num_layers,
            num_attention_heads=num_attention_heads,
            intermediate_size=intermediate_size,
            activation=activation,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            initializer_range=initializer_range,
            epsilon=epsilon,
            name="bert",
        )
        sequence_output, pooled_output, hidden_states, attention_weights = bert_model(
            input_ids, segment_ids, attention_mask
        )

        cls_embedding = tf.keras.layers.Lambda(lambda x: x[:, 0, :], name="cls")(sequence_output)
        pooler_embedding = tf.keras.layers.Lambda(lambda x: x, name="pooler")(pooled_output)
        avg_embedding = tf.keras.layers.Lambda(lambda x: tf.reduce_mean(x, axis=1), name="avg")(sequence_output)
        all_hidden_states = tf.keras.layers.Lambda(lambda x: x, name="hidden_states")(hidden_states)
        all_attention_weights = tf.keras.layers.Lambda(lambda x: x, name="attention_weights")(attention_weights)
        super().__init__(
            inputs=[input_ids, segment_ids, attention_mask],
            outputs=[cls_embedding, pooler_embedding, avg_embedding, all_hidden_states, all_attention_weights],
            **kwargs
        )

        self.bert_model = bert_model



import json
import logging
from collections import namedtuple
from typing import List

import scipy.stats
import tensorflow as tf
from tensorflow import keras
from tokenizers import BertWordPieceTokenizer

ExampleForSpearman = namedtuple("ExampleForSpearman", ["sentence_a", "sentence_b", "label"])


class SpearmanForSentenceEmbedding(tf.keras.callbacks.Callback):
    """Spearman correlation for sentence embedding."""

    @classmethod
    def from_jsonl_files(cls, input_files, vocab_file, **kwargs):
        if isinstance(input_files, str):
            input_files = [input_files]
        examples = []
        for f in input_files:
            with open(f, mode="rt", encoding="utf-8") as fin:
                for line in fin:
                    line = line.strip()
                    if not line:
                        continue
                    instance = json.loads(line)
                    sentence_a = instance[kwargs.pop("sentence_a_key", "sentence_a")]
                    sentence_b = instance[kwargs.pop("sentence_b_key", "sentence_b")]
                    label = instance[keras.pop("label_key", "label")]
                    examples.append(
                        ExampleForSpearman(
                            sentence_a=sentence_a,
                            sentence_b=sentence_b,
                            label=label,
                        )
                    )
        tokenizer = BertWordPieceTokenizer.from_file(vocab_file, lowercase=kwargs.pop("do_lower_case", True))
        return cls(examples=examples, tokenzer=tokenizer, **kwargs)

    def __init__(self, examples: List[ExampleForSpearman], tokenizer: BertWordPieceTokenizer, batch_size=32, **kwargs):
        super().__init__()
        self.examples = examples
        self.tokenizer = tokenizer
        self.batch_size = batch_size

    def on_epoch_end(self, epoch, logs):
        a_sentences = [x.sentence_a for x in self.examples]
        b_sentences = [x.sentence_b for x in self.examples]
        labels = [x.label for x in self.examples]
        a_dataset = self._build_inputs(a_sentences)
        b_dataset = self._build_inputs(b_sentences)
        a_embeddings = self.model.predict(a_dataset)
        b_embeddings = self.model.predict(b_dataset)
        correlation = self._report(a_embeddings, b_embeddings, labels)
        logging.info("No.%4d epoch Spearman Correlation: %.4f", epoch + 1, correlation)
        tf.summary.scalar("SpearmanCorrelation", correlation, step=epoch)

    def _report(self, a_embeddings, b_embeddings, labels):
        y_pred, y_true = [], []
        for a, b, l in zip(a_embeddings, b_embeddings, labels):
            norm_a = tf.linalg.normalize(a, axis=-1)[0]
            norm_b = tf.linalg.normalize(b, axis=-1)[0]
            sim = tf.reduce_sum(norm_a * norm_b, axis=-1)
            y_pred.append(sim.numpy())
            y_true.append(l)
        spearman = scipy.stats.spearmanr(y_pred, y_true)
        # print('spearman: ', spearman)
        return spearman.correlation

    def _build_inputs(self, sentences):
        input_ids, segment_ids, attention_mask = [], [], []
        for sent in sentences:
            encoding = self.tokenizer.encode(sent)
            input_ids.append(encoding.ids)
            segment_ids.append(encoding.type_ids)
            attention_mask.append(encoding.attention_mask)
        padded_input_ids, padded_segment_ids, padded_attention_mask = [], [], []
        maxlen = max([len(x) for x in input_ids])
        for ids, type_ids, mask in zip(input_ids, segment_ids, attention_mask):
            padded_input_ids.append(ids + [0] * (maxlen - len(ids)))
            padded_segment_ids.append(type_ids + [0] * (maxlen - len(type_ids)))
            padded_attention_mask.append(mask + [0] * (maxlen - len(mask)))

        input_ids = tf.constant(padded_input_ids, shape=(len(padded_input_ids), maxlen), dtype=tf.int32)
        segment_ids = tf.constant(padded_segment_ids, shape=(len(padded_segment_ids), maxlen), dtype=tf.int32)
        attention_mask = tf.constant(padded_attention_mask, shape=(len(padded_attention_mask), maxlen), dtype=tf.int32)
        dataset = tf.data.Dataset.zip(
            (
                tf.data.Dataset.from_tensor_slices(input_ids),
                tf.data.Dataset.from_tensor_slices(segment_ids),
                tf.data.Dataset.from_tensor_slices(attention_mask),
            )
        )
        dataset = dataset.batch(self.batch_size)
        dataset = dataset.map(lambda a, b, c: {"input_ids": a, "segment_ids": b, "attention_mask": c})
        return dataset



import json
import unittest

from tokenizers import BertWordPieceTokenizer
from transformers_keras.sentence_embedding.simcse_dataset import SimCSEDataset, SimCSEExample


class SimCSEDatasetTest(unittest.TestCase):
    """SimCSE dataset tests."""

    def setUp(self) -> None:
        self.tokenizer = BertWordPieceTokenizer.from_file("testdata/vocab.bert.txt")

    def _read_examples(self, input_files):
        examples = []
        for f in input_files:
            with open(f, mode="rt", encoding="Utf-8") as fin:
                for line in fin:
                    instance = json.loads(line)
                    anc_encoding = self.tokenizer.encode(instance["sequence"])
                    pos_encoding = self.tokenizer.encode(instance["pos_sequence"])
                    neg_encoding = self.tokenizer.encode(instance["neg_sequence"])
                    examples.append(
                        SimCSEExample(
                            sequence=instance["sequence"],
                            input_ids=anc_encoding.ids,
                            segment_ids=anc_encoding.type_ids,
                            attention_mask=anc_encoding.attention_mask,
                            pos_sequence=instance["pos_sequence"],
                            pos_input_ids=pos_encoding.ids,
                            pos_segment_ids=pos_encoding.type_ids,
                            pos_attention_mask=pos_encoding.attention_mask,
                            neg_sequence=instance["neg_sequence"],
                            neg_input_ids=neg_encoding.ids,
                            neg_segment_ids=neg_encoding.type_ids,
                            neg_attention_mask=neg_encoding.attention_mask,
                        )
                    )
        return examples

    def test_unsup_dataset_examples(self):
        examples = self._read_examples(["testdata/simcse.jsonl"])
        dataset = SimCSEDataset.from_examples(
            examples,
            with_pos_sequence=False,
            with_neg_sequence=False,
            batch_size=2,
        )
        print()
        print(next(iter(dataset)))

        SimCSEDataset.examples_to_tfrecord(
            examples,
            ["testdata/simcse.unsup.tfrecord"],
            with_pos_sequence=False,
            with_neg_sequence=False,
        )
        dataset = SimCSEDataset.from_tfrecord_files(
            ["testdata/simcse.unsup.tfrecord"] * 4,
            batch_size=2,
            with_pos_sequence=False,
            with_neg_sequence=False,
        )
        print()
        print(next(iter(dataset)))

    def test_unsup_dataset_jsonl(self):
        # test from jsonl
        dataset = SimCSEDataset.from_jsonl_files(
            "testdata/simcse.jsonl",
            "testdata/vocab.bert.txt",
            batch_size=2,
            with_pos_sequence=False,
            with_neg_sequence=False,
        )
        print()
        print(next(iter(dataset)))

        # test jsonl to tfrecord
        SimCSEDataset.jsonl_to_tfrecord(
            "testdata/simcse.jsonl",
            "testdata/vocab.bert.txt",
            ["testdata/simcse.unsup.tfrecord"],
            with_pos_sequence=False,
            with_neg_sequence=False,
        )
        dataset = SimCSEDataset.from_tfrecord_files(
            ["testdata/simcse.unsup.tfrecord"] * 4,
            batch_size=4,
            with_pos_sequence=False,
            with_neg_sequence=False,
        )
        print()
        print(next(iter(dataset)))

    def test_supervised_dataset_examples(self):
        examples = self._read_examples(["testdata/simcse.jsonl"])
        dataset = SimCSEDataset.from_examples(
            examples,
            with_pos_sequence=True,
            with_neg_sequence=False,
            batch_size=2,
        )
        print()
        print(next(iter(dataset)))

        SimCSEDataset.examples_to_tfrecord(
            examples,
            ["testdata/simcse.sup.tfrecord"],
            with_pos_sequence=True,
            with_neg_sequence=False,
        )
        dataset = SimCSEDataset.from_tfrecord_files(
            ["testdata/simcse.sup.tfrecord"] * 4,
            batch_size=4,
            with_pos_sequence=True,
            with_neg_sequence=False,
        )
        print()
        print(next(iter(dataset)))

    def test_supervised_dataset_jsonl(self):
        # test from jsonl
        dataset = SimCSEDataset.from_jsonl_files(
            "testdata/simcse.jsonl",
            "testdata/vocab.bert.txt",
            batch_size=2,
            with_pos_sequence=True,
            with_neg_sequence=False,
        )
        print()
        print(next(iter(dataset)))

        # test jsonl to tfrecord
        SimCSEDataset.jsonl_to_tfrecord(
            "testdata/simcse.jsonl",
            "testdata/vocab.bert.txt",
            ["testdata/simcse.sup.tfrecord"],
            with_pos_sequence=True,
            with_neg_sequence=False,
        )
        dataset = SimCSEDataset.from_tfrecord_files(
            ["testdata/simcse.sup.tfrecord"] * 4,
            batch_size=4,
            with_pos_sequence=True,
            with_neg_sequence=False,
        )
        print()
        print(next(iter(dataset)))

    def test_hardneg_dataset_examples(self):
        examples = self._read_examples(["testdata/simcse.jsonl"])
        dataset = SimCSEDataset.from_examples(
            examples,
            with_pos_sequence=False,
            with_neg_sequence=True,
            batch_size=2,
        )
        print()
        print(next(iter(dataset)))

        SimCSEDataset.examples_to_tfrecord(
            examples,
            ["testdata/simcse.hardneg.tfrecord"],
            with_pos_sequence=False,
            with_neg_sequence=True,
        )
        dataset = SimCSEDataset.from_tfrecord_files(
            ["testdata/simcse.hardneg.tfrecord"] * 4,
            batch_size=4,
            with_pos_sequence=False,
            with_neg_sequence=True,
        )
        print()
        print(next(iter(dataset)))

    def test_hardneg_dataset_jsonl(self):
        # test from jsonl
        dataset = SimCSEDataset.from_jsonl_files(
            "testdata/simcse.jsonl",
            "testdata/vocab.bert.txt",
            batch_size=2,
            with_pos_sequence=False,
            with_neg_sequence=True,
        )
        print()
        print(next(iter(dataset)))

        # test jsonl to tfrecord
        SimCSEDataset.jsonl_to_tfrecord(
            "testdata/simcse.jsonl",
            "testdata/vocab.bert.txt",
            ["testdata/simcse.hardneg.tfrecord"],
            with_pos_sequence=False,
            with_neg_sequence=True,
        )
        dataset = SimCSEDataset.from_tfrecord_files(
            ["testdata/simcse.hardneg.tfrecord"] * 4,
            batch_size=4,
            with_pos_sequence=False,
            with_neg_sequence=True,
        )
        print()
        print(next(iter(dataset)))


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from transformers_keras.sequence_classification.models import (
    AlbertForSequenceClassification,
    BertForSequenceClassification,
)

CHINESE_BERT_PATH = os.environ["CHINESE_BERT_PATH"]


class SequenceClassificationModelsTest(unittest.TestCase):
    """Sequence classificaton models tests"""

    def test_bert_for_sequence_classification(self):
        m = BertForSequenceClassification()
        m.summary()
        for w in m.trainable_weights:
            print(w.name)

    def test_bert_for_sequence_classification_from_pretrained(self):
        m = BertForSequenceClassification.from_pretrained(
            os.path.join(CHINESE_BERT_PATH, "chinese_roberta_wwm_ext_L-12_H-768_A-12"),
            override_params={"num_labels": 2},
        )
        m.summary()
        for w in m.trainable_weights:
            print(w.name)
        for w in m.trainable_weights:
            print(w.numpy())

    def test_albert_for_sequence_classification(self):
        m = AlbertForSequenceClassification()
        m.summary()
        for w in m.trainable_weights:
            print(w.name)

    def test_albert_for_sequence_classification_from_pretrained(self):
        m = AlbertForSequenceClassification.from_pretrained(
            os.path.join(os.environ["PRETRAINED_MODE_PATH"], "albert_base_zh"),
            override_params={"num_labels": 2},
        )
        m.summary()
        for w in m.trainable_weights:
            print(w.name)
        for w in m.trainable_weights:
            print(w.numpy())


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from transformers_keras.question_answering.models import (
    AlbertForQuestionAnswering, BertForQuestionAnswering)

CHINESE_BERT_PATH = os.environ['CHINESE_BERT_PATH']


class QuestionAnsweringTest(unittest.TestCase):

    def test_build_bert_for_qa_model(self):
        m = BertForQuestionAnswering()
        m.summary()
        for w in m.trainable_weights:
            print(w.name)

        m.save('models/bert-for-qa/1')

    def test_load_bert_for_qa_pretrained_model(self):
        m = BertForQuestionAnswering.from_pretrained(
            os.path.join(CHINESE_BERT_PATH, 'chinese_roberta_wwm_ext_L-12_H-768_A-12'),
            override_params={'num_labels': 2},
        )
        m.summary()
        for w in m.trainable_weights:
            print(w.name)
        for w in m.trainable_weights:
            print(w.numpy())

        m.save('models/bert-for-qa/2')

    def test_build_albetr_for_qa_model(self):
        m = AlbertForQuestionAnswering()
        m.summary()
        for w in m.trainable_weights:
            print(w.name)

    def test_load_albert_for_qa_pretrained_model(self):
        m = AlbertForQuestionAnswering.from_pretrained(
            os.path.join(os.environ['PRETRAINED_MODE_PATH'], 'albert_base_zh'),
            override_params={'num_labels': 2},
        )
        m.summary()
        for w in m.trainable_weights:
            print(w.name)
        for w in m.trainable_weights:
            print(w.numpy())


if __name__ == "__main__":
    unittest.main()



import os
import unittest

import tensorflow as tf

from transformers_keras.token_classification.crf_models import (
    AlertCRFForTokenClassification,
    BertCRFForTokenClassification,
    CRFModel,
)


class CRFModelsTest(unittest.TestCase):
    """CRF models test."""

    def test_crf_model(self):
        sequence_input = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name="sequence_input")
        outputs = tf.keras.layers.Embedding(21128, 128)(sequence_input)
        outputs = tf.keras.layers.Dense(256)(outputs)
        base = tf.keras.Model(inputs=sequence_input, outputs=outputs)
        m = CRFModel(base, 5)
        m.summary()
        m.save("models/crf-model/1")

    def test_bert_crf_for_token_classification(self):
        m = BertCRFForTokenClassification(4)
        m.summary()
        m.save("models/bert-crf-for-token-classification/1", signatures=m.forward)

        bert_path = os.path.join(os.environ["CHINESE_BERT_PATH"], "chinese_roberta_wwm_ext_L-12_H-768_A-12")
        m = BertCRFForTokenClassification.from_pretrained(bert_path, override_params={"num_labels": 4})
        m.summary()
        m.save("models/bert-crf-for-token-classification/2", signatures=m.forward)

    def test_load_bert_crf_for_token_classification(self):
        # load from saved model
        m = tf.saved_model.load("models/bert-crf-for-token-classification/2")
        model = m.signatures["serving_default"]
        print(model.structured_input_signature)
        print(model.structured_outputs)

    def test_albert_crf_for_token_classification(self):
        m = AlertCRFForTokenClassification(4)
        m.summary()
        m.save("models/albert-crf-for-token-classification/1", signatures=m.forward)

        albert_path = os.path.join(os.environ["GOOGLE_ALBERT_PATH"], "albert-base-zh")
        m = AlertCRFForTokenClassification.from_pretrained(albert_path, override_params={"num_labels": 4})
        m.summary()
        m.save("models/albert-crf-for-token-classification/2", signatures=m.forward)

    def test_load_albert_crf_for_token_classification(self):
        # load from saved model
        m = tf.saved_model.load("models/albert-crf-for-token-classification/2")
        model = m.signatures["serving_default"]
        print(model.structured_input_signature)
        print(model.structured_outputs)

    def test_bert_predict(self):
        m = BertCRFForTokenClassification(4)
        input_ids, segment_ids, attention_mask = m.dummy_inputs()
        dataset = tf.data.Dataset.zip(
            (
                tf.data.Dataset.from_tensor_slices([input_ids]),
                tf.data.Dataset.from_tensor_slices([segment_ids]),
                tf.data.Dataset.from_tensor_slices([attention_mask]),
            )
        )
        dataset = dataset.map(lambda a, b, c: ({"input_ids": a, "segment_ids": b, "attention_mask": c}, None))
        pred = m.predict(dataset)
        print("\npred: ", pred)


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from transformers_keras.sentence_embedding.bert_embedding import BertForSentenceEmbedding


class BertEmbeddingTest(unittest.TestCase):
    """Bert for sentence embedding tests."""

    def test_build_bert_for_sentence_embedding(self):
        m = BertForSentenceEmbedding()
        m.summary()
        m.save("models/bert-for-sentence-embedding/1")

    def test_bert_for_sentence_embedding_load_pretrained(self):
        bert_path = os.path.join(os.environ["CHINESE_BERT_PATH"], "chinese_roberta_wwm_ext_L-12_H-768_A-12")
        m = BertForSentenceEmbedding.from_pretrained(bert_path)
        m.summary()
        m.save("models/bert-for-sentence-embedding/2")


if __name__ == "__main__":
    unittest.main()



import logging
from typing import List

import numpy as np
import tensorflow as tf
from seqeval.metrics import classification_report
from transformers_keras.token_classification.dataset import TokenClassificationDataset, TokenClassificationExample
from transformers_keras.token_classification.tokenizer import TokenClassificationLabelTokenizer


class SeqEvalForTokenClassification(tf.keras.callbacks.Callback):
    """Seqeval for token classification"""

    @classmethod
    def from_conll_files(cls, input_files, vocab_file, label_vocab_file, sep="\t", **kwargs):
        examples = TokenClassificationDataset.conll_to_examples(
            input_files, vocab_file, label_vocab_file, sep=sep, **kwargs
        )
        label_tokenizer = TokenClassificationLabelTokenizer.from_file(
            label_vocab_file, o_token=kwargs.pop("o_token", "O")
        )
        return cls(examples=examples, label_tokenizer=label_tokenizer, **kwargs)

    def __init__(
        self,
        examples: List[TokenClassificationExample],
        label_tokenizer: TokenClassificationLabelTokenizer,
        batch_size=32,
        **kwargs
    ):
        super().__init__()
        self.examples = examples
        self.label_tokenizer = label_tokenizer
        self.batch_size = batch_size

    def on_epoch_end(self, epoch, logs):
        input_ids, segment_ids, attention_mask, labels = self._build_inputs()
        dataset = tf.data.Dataset.zip(
            (
                tf.data.Dataset.from_tensor_slices(input_ids),
                tf.data.Dataset.from_tensor_slices(segment_ids),
                tf.data.Dataset.from_tensor_slices(attention_mask),
            )
        ).batch(self.batch_size)
        dataset = dataset.map(lambda a, b, c: {"input_ids": a, "segment_ids": b, "attention_mask": c})
        pred_ids = self.model.predict(dataset)
        y_preds, y_trues = [], []
        for y_pred, y_true in zip(pred_ids, labels):
            y_pred = np.argmax(y_pred[: len(y_true)], axis=-1)
            y_pred = self.label_tokenizer.ids_to_labels(y_pred.tolist())
            y_preds.append(y_pred)
            y_trues.append(y_true)
        report = classification_report(y_trues, y_preds)
        logging.info("SeqEval Reports at epoch {}:\n {}".format(epoch + 1, report))

    def _build_inputs(self):
        input_ids, segment_ids, attention_mask, labels = [], [], [], []
        maxlen = max([len(e.tokens) for e in self.examples])
        for e in self.examples:
            _input_ids = e.input_ids + [0] * (maxlen - len(e.input_ids))
            _segment_ids = e.segment_ids + [0] * (maxlen - len(e.segment_ids))
            _attention_mask = e.attention_mask + [0] * (maxlen - len(e.attention_mask))
            input_ids.append(_input_ids)
            segment_ids.append(_segment_ids)
            attention_mask.append(_attention_mask)
            labels.append(e.labels)
        input_ids = tf.constant(input_ids, shape=(len(input_ids), maxlen), dtype=tf.int32)
        segment_ids = tf.constant(segment_ids, shape=(len(segment_ids), maxlen), dtype=tf.int32)
        attention_mask = tf.constant(attention_mask, shape=(len(attention_mask), maxlen), dtype=tf.int32)
        return input_ids, segment_ids, attention_mask, labels
